/*
 * Copyright 2011 E-Government Innovation Center (EGIZ)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package at.gv.zustellung.zkopf.client;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.net.ssl.SSLContext;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.Handler;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.util.LaxHostNameVerifier;
import at.gv.util.LoggingHandler;
import at.gv.util.ex.EgovUtilException;
import at.gv.zustellung.zkopf.ZuseKopfException;
import at.gv.zustellung.zkopf.config.ZuseKopfConfiguration;
import at.gv.zustellung.zkopf.wsdl.BulkPortType;
import at.gv.zustellung.zkopf.wsdl.BulkService;
import at.gv.zustellung.zkopf.xsd.zkopf.BulkAnswerType;
import at.gv.zustellung.zkopf.xsd.zkopf.BulkQueryType;

import com.sun.xml.ws.developer.JAXWSProperties;

public class BulkQueryClient {

	@Resource
  WebServiceContext wsContext;

	private ZuseKopfConfiguration config = null;
	private BulkPortType port = null;
	
	Logger log = LoggerFactory.getLogger(BulkQueryClient.class);
	
	public BulkQueryClient(ZuseKopfConfiguration config) {
		if (config == null) {
			throw new NullPointerException("Parameter config must not be null.");
		}
		this.config = config;
		
		URL url = BulkQueryClient.class.getResource("/wsdl/zkopf.wsdl");
		BulkService service = new BulkService(url, new QName("http://reference.e-government.gv.at/namespaces/zustellung/kopf/wsdl", "BulkService"));
		this.port = service.getBulkPort();
	}
	
  @SuppressWarnings("unchecked")
  public BulkAnswerType sendBulkQueryRequest(String bulkQueryServiceURL, BulkQueryType request) throws BulkQueryClientException {
    
  	// verify that parameters are not null
  	if (bulkQueryServiceURL == null) {
  		throw new NullPointerException("Parameter 'serviceURL' must not be null.");
  	}
  	log.trace("Bulk query request URL: " + bulkQueryServiceURL);
  	if (request == null) {
  		throw new NullPointerException("Parameter 'request' must not be null.");
  	}
  	
    try {
    
	   // set endpoint address
  		BindingProvider bindingProvider = (BindingProvider) this.port;
      bindingProvider.getRequestContext().put(
              BindingProvider.ENDPOINT_ADDRESS_PROPERTY, bulkQueryServiceURL);
      
    	// add trace log
      log.trace("Adding JAX-WS request/response trace handler.");
    	List<Handler> handlerList = bindingProvider.getBinding().getHandlerChain();
    	if (handlerList == null) {
	      handlerList = new ArrayList();
      }
	    LoggingHandler loggingHandler = new LoggingHandler();
	    handlerList.add(loggingHandler);
	    bindingProvider.getBinding().setHandlerChain(handlerList);
	    
	    Map<String, Object> requestContext = ((BindingProvider) this.port).getRequestContext();
      
      // initialize ssl
      if (bulkQueryServiceURL.toLowerCase().startsWith("https")) {
      	log.trace("Using ssl for bulk query request.");
				SSLContext sslContext = config.getSSLConfiguration().getSSLContext(false);
				if (sslContext == null) {
					throw new BulkQueryClientException("SSL context from configuration is empty. Please configure an SSL context in the configuration first.");
				}
				requestContext.put(JAXWSProperties.SSL_SOCKET_FACTORY, sslContext.getSocketFactory());
				
		    if (this.config.getSSLConfiguration().useLaxHostNameVerifier()) {
		    	log.trace("LaxHostnameVerifier enabled. This setting is not recommended to use.");
	      	requestContext.put(JAXWSProperties.HOSTNAME_VERIFIER, new LaxHostNameVerifier());
	      }
      }

			return this.port.bulkOperation(request);
      
    } catch (ZuseKopfException e) {
    	throw new BulkQueryClientException(e);
    } catch (EgovUtilException e) {
    	throw new BulkQueryClientException(e);
    }
  }
  
}
