/*
 * Copyright 2011 E-Government Innovation Center (EGIZ)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package at.gv.zustellung.zkopf.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;

import javax.net.ssl.SSLContext;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.util.MiscUtil;
import at.gv.util.ex.EgovUtilException;
import at.gv.zustellung.zkopf.ZuseKopfException;
import at.gv.zustellung.zkopf.config.ZuseKopfConfiguration;
import at.gv.zustellung.zkopf.xsd.zkopf.PushResponseType;

public class PushClient {

	Logger log = LoggerFactory.getLogger(PushClient.class);
	
	private ZuseKopfConfiguration config = null;
	
	public PushClient(ZuseKopfConfiguration config) {
		this.config = config;
	}
	
	public void initialize(ZuseKopfConfiguration config) {
		this.config = config;
	}
	
	@SuppressWarnings("unchecked")
  public PushResponseType push(String serviceURL, InputStream request) throws PushClientException {
		try {
			log.debug("Executing push request.");
			if (config == null) {
				throw new PushClientException("Please configure the module first.");
			}
			if (serviceURL == null) {
				throw new NullPointerException("Parameter 'serviceURL' must not be null.");
			}
			if (request == null) {
				throw new NullPointerException("Parameter 'request' must not be null.");
			}
			
			// do we connect to zusekopf using SSL?
			boolean useSSL = serviceURL.toLowerCase().startsWith("https");
			
			log.trace("Standard push URL: " + serviceURL);
			
			URI uri = new URI(serviceURL);
			HttpPost httpPost = new HttpPost(uri);

			
			// set content type
			httpPost.setHeader("Content-Type", "application/directory; charset=ISO-8859-1");

			// dump input stream						
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			MiscUtil.copyStream(request, bos);
			ByteArrayInputStream bis = new ByteArrayInputStream(bos.toByteArray());
			
			// ldif content
			InputStreamEntity entity = new InputStreamEntity(bis, bos.size());
			httpPost.setEntity(entity);
			
			SchemeRegistry schemeRegistry = new SchemeRegistry();
			// configure used protocol for client
			log.trace("SSL enabled: " + useSSL);

			if (useSSL) {
				
				SSLContext sslContext = config.getSSLPushConfiguration().getSSLContext(false);
				if (sslContext == null) {
					throw new PushClientException("SSL context from configuration is empty. Please configure an SSL context in the configuration first.");
				}
				
				SSLSocketFactory sslFactory = new SSLSocketFactory(sslContext);
				// check for lax hostname verification
				if (config.getSSLPushConfiguration().useLaxHostNameVerifier()) {
					log.trace("LaxHostnameVerifier enabled. This setting is not recommended to use.");
					sslFactory.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
				}
				int port = uri.getPort() > 0 ? uri.getPort() : 443;
				schemeRegistry.register(
				        new Scheme("https", sslFactory, port));
			} else {
				log.trace("using standard http");
				int port = uri.getPort() > 0 ? uri.getPort() : 80;
				schemeRegistry.register(
				        new Scheme("http", PlainSocketFactory.getSocketFactory(), port));
			}
			HttpParams params = new BasicHttpParams();
			ClientConnectionManager cm = new ThreadSafeClientConnManager(params, schemeRegistry);
			HttpClient httpClient = new DefaultHttpClient(cm, params);

			log.trace("Executing query.");
			HttpResponse httpResponse = httpClient.execute(httpPost);
			log.trace("HTTP response code: " + httpResponse.getStatusLine().getStatusCode());
			HttpEntity httpEntity = httpResponse.getEntity();
			if (httpEntity == null) {
				log.warn("HTTP entity is null.");
				throw new PushClientException("No http entity in response found.");
			}

			// try to unmarshall
			JAXBContext jaxbCtx = JAXBContext.newInstance(PushResponseType.class.getPackage().getName());
			Unmarshaller um = jaxbCtx.createUnmarshaller();
			bos = new ByteArrayOutputStream();
			
			// copy input stream
			MiscUtil.copyStream(httpEntity.getContent(), bos);
			
			JAXBElement<PushResponseType> pushResponseElement = (JAXBElement<PushResponseType>) um.unmarshal(new ByteArrayInputStream(bos.toByteArray()));

			log.trace(bos.toString());
			
			return pushResponseElement.getValue();
		} catch(URISyntaxException e) {
			throw new PushClientException(e);
		} catch (ClientProtocolException e) {
			throw new PushClientException(e);
    } catch (IOException e) {
    	throw new PushClientException(e);
    } catch (JAXBException e) {
    	throw new PushClientException(e);
    } catch (ZuseKopfException e) {
    	throw new PushClientException(e);
    } catch (EgovUtilException e) {
    	throw new PushClientException(e);
    }
	}
	
}
