/*
 * Copyright 2011 E-Government Innovation Center (EGIZ)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package at.gv.zustellung.zkopf.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.net.ssl.SSLContext;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.util.MiscUtil;
import at.gv.util.ex.EgovUtilException;
import at.gv.zustellung.zkopf.ZuseKopfException;
import at.gv.zustellung.zkopf.config.ZuseKopfConfiguration;
import at.gv.zustellung.zkopf.query.StandardQuery;
import at.gv.zustellung.zkopf.xsd.zkopf.StdAnswerType;
import at.gv.zustellung.zkopf.xsd.zkopf.ZuseType;

public class StdQueryClient {

	Logger log = LoggerFactory.getLogger(StdQueryClient.class);
	
	private ZuseKopfConfiguration config = null;
	
	public StdQueryClient(ZuseKopfConfiguration config) {
		this.config = config;
	}
	
	public void initialize(ZuseKopfConfiguration config) {
		this.config = config;
	}
	
	public static ZuseType selectRandomZuseServer(StdAnswerType zkopfAnswer) {
		if (zkopfAnswer == null) {
			return null;
		}
		if (zkopfAnswer.getSuccess() == null) {
			return null;
		}
		// search first for servers with certificate
		ArrayList<ZuseType> serversWithCert = new ArrayList<ZuseType>();
		for (ZuseType server : zkopfAnswer.getSuccess().getServer()) {
			if (server.getX509() != null) {
				serversWithCert.add(server);
			}
		}
		if (serversWithCert.size() > 0) {
			return selectRandomServer(serversWithCert);
		}
		// ok, we have only servers without encryption certificate
		return selectRandomServer(zkopfAnswer.getSuccess().getServer());
	}
	
	private static ZuseType selectRandomServer(List<ZuseType> servers) {
		Random random = new Random(System.currentTimeMillis());
		return servers.get(random.nextInt(servers.size()));
	}
	
	@SuppressWarnings("unchecked")
  public StdAnswerType stdQuery(String serviceURL, StandardQuery query) throws StdQueryClientException {
		try {
			log.debug("Executing standard query.");
			if (config == null) {
				throw new StdQueryClientException("Please configure the module first.");
			}
			if (serviceURL == null) {
				throw new NullPointerException("Parameter 'serviceURL' must not be null.");
			}
			
			// do we connect to zusekopf using SSL?
			boolean useSSL = serviceURL.toLowerCase().startsWith("https");
			
			log.trace("Standard query URL: " + serviceURL);
			if (!serviceURL.endsWith("?")) {
				serviceURL += "?";
			}
			// add parameters
			if (query.getQuery() == null) {
				throw new NullPointerException("Query must not be null.");
			}
			
			List<NameValuePair> qParams = new ArrayList<NameValuePair>();
			for (String paramName : query.getQuery().keySet()) {
				qParams.add(new BasicNameValuePair(paramName, query.getQuery().get(paramName)));
			}
			URI uri = new URI(serviceURL + URLEncodedUtils.format(qParams, "UTF-8"));
			log.trace("ZuseKopf request URI: " + uri);
			
			HttpGet httpGet = new HttpGet(uri);
			
			SchemeRegistry schemeRegistry = new SchemeRegistry();
			// configure used protocol for client
			log.trace("SSL enabled: " + useSSL);

			if (useSSL) {
				
				SSLContext sslContext = config.getSSLConfiguration().getSSLContext(false);
				if (sslContext == null) {
					throw new StdQueryClientException("SSL context from configuration is empty. Please configure an SSL context in the configuration first.");
				}
				
				SSLSocketFactory sslFactory = new SSLSocketFactory(sslContext);
				// check for lax hostname verification
				if (config.getSSLConfiguration().useLaxHostNameVerifier()) {
					log.trace("LaxHostnameVerifier enabled. This setting is not recommended to use.");
					sslFactory.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
				}
				int port = uri.getPort() > 0 ? uri.getPort() : 443;
				schemeRegistry.register(
				        new Scheme("https", sslFactory, port));
			} else {
				log.trace("using standard http");
				int port = uri.getPort() > 0 ? uri.getPort() : 80;
				schemeRegistry.register(
				        new Scheme("http", PlainSocketFactory.getSocketFactory(), port));
			}
			HttpParams params = new BasicHttpParams();
			ClientConnectionManager cm = new ThreadSafeClientConnManager(params, schemeRegistry);
			HttpClient httpClient = new DefaultHttpClient(cm, params);

			log.trace("Executing query.");
			HttpResponse httpResponse = httpClient.execute(httpGet);
			log.trace("HTTP response code: " + httpResponse.getStatusLine().getStatusCode());
			HttpEntity httpEntity = httpResponse.getEntity();
			if (httpEntity == null) {
				log.warn("HTTP entity is null.");
				throw new StdQueryClientException("No http entity in response found.");
			}

			// try to unmarshall
			JAXBContext jaxbCtx = JAXBContext.newInstance(StdAnswerType.class.getPackage().getName());
			Unmarshaller um = jaxbCtx.createUnmarshaller();
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			
			// copy input stream
			MiscUtil.copyStream(httpEntity.getContent(), bos);
			
			JAXBElement<StdAnswerType> stdAnswerElement = (JAXBElement<StdAnswerType>) um.unmarshal(new ByteArrayInputStream(bos.toByteArray()));

			log.trace(bos.toString());
			
			
			return stdAnswerElement.getValue();
		} catch(URISyntaxException e) {
			throw new StdQueryClientException(e);
		} catch (ClientProtocolException e) {
			throw new StdQueryClientException(e);
    } catch (IOException e) {
    	throw new StdQueryClientException(e);
    } catch (JAXBException e) {
    	throw new StdQueryClientException(e);
    } catch (ZuseKopfException e) {
    	throw new StdQueryClientException(e);
    } catch (EgovUtilException e) {
    	throw new StdQueryClientException(e);
    }
	}
	
}
