/*
 * Copyright 2011 E-Government Innovation Center (EGIZ)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package at.gv.zustellung.zkopf.query;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.zustellung.zkopf.ZuseKopfConstants;
import at.gv.zustellung.zkopf.client.PostalAddress;
import at.gv.zustellung.zkopf.client.StdQueryClientException;

/**
 * 
 * @author Arne Tauber <arne.tauber@iaik.tugraz.at>
 */
public class DefaultStandardQuery implements StandardQuery {

	private final static List<String> ALLOWED_IDENTIFIERS = Arrays.asList(ZuseKopfConstants.SOURCEPIN_TYPE_STD_QUERY_XFN, 
			ZuseKopfConstants.SOURCEPIN_TYPE_STD_QUERY_XZVR, ZuseKopfConstants.SOURCEPIN_TYPE_STD_QUERY_XERSB);
	
	private Map<String, String> query;
	Logger log = LoggerFactory.getLogger(DefaultStandardQuery.class);

	public DefaultStandardQuery(Map<String, String> query) {
		if (query == null) {
			throw new NullPointerException("Argument 'query' must not be null.");
		}
		this.query = query;
	}
	
	public DefaultStandardQuery(String gvZbPK, String customerNumber) {
		if (gvZbPK == null) {
			throw new NullPointerException("Parameter 'gvZbPK' must not be null.");
		}
		query = new HashMap<String, String>();
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_GVZBPK, gvZbPK);
		if (customerNumber != null) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_CUSTOMER_NUMBER, customerNumber);
		}
	}
	
	public DefaultStandardQuery(String sourcePin, String sourcePinType, boolean isPrivate, String customerNumber) throws StdQueryClientException {
		if (sourcePin == null) {
			throw new NullPointerException("Parameter 'sourcePin' must not be null.");
		}
		query = new HashMap<String, String>();
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_GVSOURCEPIN, sourcePin);
		
		if (sourcePinType != null) {
			if (!ALLOWED_IDENTIFIERS.contains(sourcePinType)) {
				throw new StdQueryClientException("Parameter " + sourcePinType + " is not a valid value. Allowed values are " + ALLOWED_IDENTIFIERS);
			}
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_GVSOURCEPIN_TYPE, sourcePinType);
		} else {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_GVSOURCEPIN_TYPE, ZuseKopfConstants.SOURCEPIN_TYPE_STD_QUERY_XFN);	
		}
		if (isPrivate) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_PRIVATE, String.valueOf(isPrivate));
		}
		if (customerNumber != null) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_CUSTOMER_NUMBER, customerNumber);
		}
	}
	
	public DefaultStandardQuery(String givenName,
			String familyName,
			String dateOfBirth,
			String emailAddress,
			String telephoneNumber,
			boolean isPrivate,
			String customerNumber) {

		initializePhysicalPerson(givenName, familyName, dateOfBirth, isPrivate, customerNumber);
		initializeEmailTelephone(emailAddress, telephoneNumber);
	}

	public DefaultStandardQuery(String givenName,
			String familyName,
			String dateOfBirth,
			PostalAddress postalAddress,
			boolean isPrivate,
			String customerNumber) {

		initializePhysicalPerson(givenName, familyName, dateOfBirth, isPrivate, customerNumber);
		initializePostalAddress(postalAddress);
	}
	
	public DefaultStandardQuery(String corporateBodyName,
			String emailAddress,
			String telephoneNumber,
			boolean isPrivate,
			String customerNumber) {

		initializeCorporateBody(corporateBodyName, isPrivate, customerNumber);
		initializeEmailTelephone(emailAddress, telephoneNumber);
	}
	
	public DefaultStandardQuery(String corporateBodyName,
			PostalAddress postalAddress,
			boolean isPrivate,
			String customerNumber) {

		initializeCorporateBody(corporateBodyName, isPrivate, customerNumber);
		initializePostalAddress(postalAddress);
	}
	
	private void initializeCorporateBody(String corporateBodyName, boolean isPrivate, String customerNumber) {
		if (corporateBodyName == null) {
			throw new NullPointerException("Parameter 'corporateBodyName' must not be null.");
		}
		query = new HashMap<String, String>();
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_CN, corporateBodyName);
		if (isPrivate) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_PRIVATE, String.valueOf(isPrivate));
		}
		if (customerNumber != null) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_CUSTOMER_NUMBER, customerNumber);
		}
	}
	
	private void initializePostalAddress(PostalAddress postalAddress) {
		
		if (postalAddress == null) {
			throw new NullPointerException("Parameter 'postalAddress' must not be null.");
		}
		
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_STREETNAME, postalAddress.getStreetName());
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_POSTALCODE, postalAddress.getPostalCode());
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_MUNICIPALITY, postalAddress.getMunicipality());
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_COUNTRYCODE, postalAddress.getCountryCode());
	}
	
	private void initializeEmailTelephone(String emailAddress,
			String telephoneNumber) {
		if (emailAddress == null && telephoneNumber == null) {
			throw new NullPointerException("Specify at least one address parameter 'emailAddress' or 'telephoneNumber'.");
		}
		if (emailAddress != null) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_MAIL, emailAddress);
		}
		if (telephoneNumber != null) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_TELEPHONE, telephoneNumber);
		}
	}
	
	private void initializePhysicalPerson(String givenName,
			String familyName,
			String dateOfBirth,
			boolean isPrivate,
			String customerNumber) {	
		
		if (givenName == null) {
			throw new NullPointerException("Parameter 'givenName' must not be null.");
		}
		if (familyName == null) {
			throw new NullPointerException("Parameter 'familyName' must not be null.");
		}
		query = new HashMap<String, String>();
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_GIVEN_NAME, givenName);
		query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_FAMILY_NAME, familyName);
		if (dateOfBirth != null) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_DATE_OF_BIRTH, dateOfBirth);
		} else {
			log.trace("Date of birth missing, query intention is for non qualitative delivery, e.g. nonRSa.");
		}
		
		
		if (isPrivate) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_PRIVATE, String.valueOf(isPrivate));
		}
		if (customerNumber != null) {
			query.put(ZuseKopfConstants.HTTP_QUERY_PARAM_CUSTOMER_NUMBER, customerNumber);
		}
	}
		  
	
	@Override
	public Map<String, String> getQuery() {
		return this.query;
	}

}
